%global srcname superlu
%global libver 5.3

Summary:    Subroutines to solve sparse linear systems
Name:       SuperLU
Version:    5.3.0
Release:    3%{?dist}
License:    BSD and GPLv2+
URL:        https://portal.nersc.gov/project/sparse/superlu/
Source0:    https://github.com/xiaoyeli/superlu/archive/v%{version}/%{srcname}-%{version}.tar.gz

Patch3000:        %{srcname}-removemc64.patch
Patch3001:        %{name}-set_soname.patch
Patch3002:        %{name}-fix_example_builds.patch
Patch3003:        SuperLU-c99.patch
Patch5000:        fix-build-error.patch

%ifarch x86_64 ppc64le aarch64
BuildRequires:    openblas-devel
%else
BuildRequires:    blas-devel
%endif
BuildRequires:    atlas-devel make cmake3 gcc gcc-gfortran

%description
SuperLU is a general purpose library for the direct solution of large, sparse,
nonsymmetric systems of linear equations. The library is written in C and is
callable from either C or Fortran program. It uses MPI, OpenMP and CUDA to
support various forms of parallelism. It supports both real and complex datatypes,
both single and double precision, and 64-bit integer indexing. The library routines
performs an LU decomposition with partial pivoting and triangular system solves through
forward and back substitution. The LU factorization routines can handle non-square
matrices but the triangular solves are performed only for square matrices. The matrix
columns may be preordered (before factorization) either through library or user
supplied routines. This preordering for sparsity is completely separate from the
factorization. Working precision iterative refinement subroutines are provided for
improved backward stability. Routines are also provided to equilibrate the system,
estimate the condition number, calculate the relative backward error, and estimate
error bounds for the refined solutions.

%package devel
Summary:    Header files and libraries for SuperLU development
Requires:    %{name} = %{version}-%{release}

%description devel 
The %{name}-devel package contains the header files
and libraries for use with %{name} package.

%package doc
Summary:    Documentation and Examples for SuperLU
BuildArch:    noarch

%description doc
The %{name}-doc package contains all the help HTML documentation.

%prep
%autosetup -n %{srcname}-%{version} -p1

rm -rf CBLAS
rm -fr SRC/mc64ad.f.bak
find . -type f | sed -e "/TESTING/d" | xargs chmod a-x

find EXAMPLE -type f | while read file
do
   [ "$(file $file | awk '{print $2}')" = ELF ] && rm $file || :
done


%build
mkdir -p build
%cmake3 -S . -B build \
   -Denable_internal_blaslib:BOOL=NO \
   -DTPL_BLAS_LIBRARIES=-lopenblas   \
   -DCMAKE_BUILD_TYPE:STRING=Release \
   -DCMAKE_INSTALL_INCLUDEDIR:PATH=include/%{name} \
   -DCMAKE_INSTALL_LIBDIR:PATH=%{_lib} \
   -DCMAKE_SKIP_RPATH:BOOL=YES -DCMAKE_SKIP_INSTALL_RPATH:BOOL=YES
%make_build -C build

export TPL_BLAS_LIBRARIES=-lopenblas \
export LDFLAGS="%{build_ldflags}"
export FFLAGS="%{build_fflags}"
export CFLAGS="%{build_cflags}"
make -C FORTRAN
make -C EXAMPLE

%install
%make_install -C build

%check
pushd build
ctest3 -V %{?_smp_mflags}
popd

pushd FORTRAN
export LD_LIBRARY_PATH=%{buildroot}%{_libdir}
./df77exm < ../EXAMPLE/g20.rua
./zf77exm < ../EXAMPLE/cg20.cua
./test_omp < ../EXAMPLE/g20.rua
make clean
popd

pushd EXAMPLE
export LD_LIBRARY_PATH=%{buildroot}%{_libdir}
./superlu
make clean
popd

%files
%license License.txt
%{_libdir}/libsuperlu.so.5
%{_libdir}/libsuperlu.so.%{libver}

%files devel
%{_includedir}/%{name}/
%{_libdir}/libsuperlu.so
%{_libdir}/cmake/%{srcname}/
%{_libdir}/pkgconfig/%{srcname}.pc

%files doc
%license License.txt
%doc DOC

%changelog
* Fri Sep 08 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 5.3.0-3
- Rebuilt for OpenCloudOS Stream 23.09

* Fri Apr 28 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 5.3.0-2
- Rebuilt for OpenCloudOS Stream 23.05

* Sun Apr 23 2023 cunshunxia <cunshunxia@tencent.com> - 5.3.0-1
- initial build
